/*
    Copyright (C) 2001-2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef SVGElementInstance_H
#define SVGElementInstance_H

namespace KSVG
{

class SVGElement;
class SVGUseElement;
class SVGElementInstanceList;
class SVGElementInstanceImpl;
class SVGElementInstance
{
public:
	SVGElementInstance();
	SVGElementInstance(const SVGElementInstance &other);
	SVGElementInstance &operator=(const SVGElementInstance &other);
	SVGElementInstance(SVGElementInstanceImpl *other);
	~SVGElementInstance();

	SVGElement correspondingElement() const;
	SVGUseElement correspondingUseElement() const;
	SVGElementInstance parentNode() const;
	SVGElementInstanceList childNodes() const;
	SVGElementInstance firstChild() const;
	SVGElementInstance lastChild() const;
	SVGElementInstance previousSibling() const;
	SVGElementInstance nextSibling() const;

	// Internal! - NOT PART OF THE SVG SPECIFICATION
	SVGElementInstanceImpl *handle() const { return impl; }

private:
	SVGElementInstanceImpl *impl;
};

};

#endif

// vim:ts=4:noet
