/* Automatically generated from impl/SVGGradientElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGGradientElementImpl__s_hashTableStrings[] = {
    "\0"
    "gradientTransform\0"
    "gradientUnits\0"
    "spreadMethod\0"
};


static const struct HashEntry SVGGradientElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 19, SVGGradientElementImpl::GradientUnits, DontDelete|ReadOnly, 0, 5 },
   { 33, SVGGradientElementImpl::SpreadMethod, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGGradientElementImpl::GradientTransform, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGGradientElementImpl::s_hashTable = { 2, 6, SVGGradientElementImpl__s_hashTableEntries, 5, SVGGradientElementImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGGradientElementImplConstructor__s_hashTableStrings[] = {
    "\0"
    "SVG_SPREADMETHOD_REFLECT\0"
    "SVG_SPREADMETHOD_UNKNOWN\0"
    "SVG_SPREADMETHOD_REPEAT\0"
    "SVG_SPREADMETHOD_PAD\0"
};


static const struct HashEntry SVGGradientElementImplConstructor__s_hashTableEntries[] = {
   { 51, KSVG::SVG_SPREADMETHOD_REPEAT, DontDelete|ReadOnly, 0, -1 },
   { 26, KSVG::SVG_SPREADMETHOD_UNKNOWN, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, KSVG::SVG_SPREADMETHOD_REFLECT, DontDelete|ReadOnly, 0, -1 },
   { 75, KSVG::SVG_SPREADMETHOD_PAD, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGGradientElementImplConstructor::s_hashTable = { 2, 5, SVGGradientElementImplConstructor__s_hashTableEntries, 5, SVGGradientElementImplConstructor__s_hashTableStrings};

} // namespace
