/* Automatically generated from impl/SVGMarkerElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGMarkerElementImpl__s_hashTableStrings[] = {
    "\0"
    "markerHeight\0"
    "markerUnits\0"
    "markerWidth\0"
    "orientAngle\0"
    "orientType\0"
    "orient\0"
    "refX\0"
    "refY\0"
};


static const struct HashEntry SVGMarkerElementImpl__s_hashTableEntries[] = {
   { 1, SVGMarkerElementImpl::MarkerHeight, DontDelete|ReadOnly, 0, 12 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 14, SVGMarkerElementImpl::MarkerUnits, DontDelete|ReadOnly, 0, -1 },
   { 50, SVGMarkerElementImpl::OrientType, DontDelete|ReadOnly, 0, 13 },
   { 68, SVGMarkerElementImpl::RefX, DontDelete|ReadOnly, 0, -1 },
   { 73, SVGMarkerElementImpl::RefY, DontDelete|ReadOnly, 0, 11 },
   { 26, SVGMarkerElementImpl::MarkerWidth, DontDelete|ReadOnly, 0, -1 },
   { 38, SVGMarkerElementImpl::OrientAngle, DontDelete|ReadOnly, 0, -1 },
   { 61, SVGMarkerElementImpl::Orient, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGMarkerElementImpl::s_hashTable = { 2, 14, SVGMarkerElementImpl__s_hashTableEntries, 11, SVGMarkerElementImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGMarkerElementImplProto__s_hashTableStrings[] = {
    "\0"
    "setOrientToAngle\0"
    "setOrientToAuto\0"
};


static const struct HashEntry SVGMarkerElementImplProto__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 18, SVGMarkerElementImpl::SetOrientToAuto, DontDelete|Function, 0, 3 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGMarkerElementImpl::SetOrientToAngle, DontDelete|Function, 0, -1 }
};

const struct HashTable SVGMarkerElementImplProto::s_hashTable = { 2, 4, SVGMarkerElementImplProto__s_hashTableEntries, 3, SVGMarkerElementImplProto__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGMarkerElementImplConstructor__s_hashTableStrings[] = {
    "\0"
    "SVG_MARKERUNITS_USERSPACEONUSE\0"
    "SVG_MARKERUNITS_STROKEWIDTH\0"
    "SVG_MARKER_ORIENT_UNKNOWN\0"
    "SVG_MARKERUNITS_UNKNOWN\0"
    "SVG_MARKER_ORIENT_ANGLE\0"
    "SVG_MARKER_ORIENT_AUTO\0"
};


static const struct HashEntry SVGMarkerElementImplConstructor__s_hashTableEntries[] = {
   { 110, KSVG::SVG_MARKER_ORIENT_ANGLE, DontDelete|ReadOnly, 0, -1 },
   { 1, KSVG::SVG_MARKERUNITS_USERSPACEONUSE, DontDelete|ReadOnly, 0, -1 },
   { 86, KSVG::SVG_MARKERUNITS_UNKNOWN, DontDelete|ReadOnly, 0, -1 },
   { 134, KSVG::SVG_MARKER_ORIENT_AUTO, DontDelete|ReadOnly, 0, -1 },
   { 32, KSVG::SVG_MARKERUNITS_STROKEWIDTH, DontDelete|ReadOnly, 0, -1 },
   { 60, KSVG::SVG_MARKER_ORIENT_UNKNOWN, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGMarkerElementImplConstructor::s_hashTable = { 2, 7, SVGMarkerElementImplConstructor__s_hashTableEntries, 7, SVGMarkerElementImplConstructor__s_hashTableStrings};

} // namespace
