/*
    Copyright (C) 2001-2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include "SVGPathSegLinetoHorizontal.h"
#include "SVGPathSegLinetoHorizontalImpl.h"

using namespace KSVG;

SVGPathSegLinetoHorizontalAbs::SVGPathSegLinetoHorizontalAbs() : SVGPathSeg()
{
	impl = new SVGPathSegLinetoHorizontalAbsImpl();
}
 
SVGPathSegLinetoHorizontalAbs::SVGPathSegLinetoHorizontalAbs(const SVGPathSegLinetoHorizontalAbs &other) : SVGPathSeg(other)
{
	impl = other.impl;
}

SVGPathSegLinetoHorizontalAbs::SVGPathSegLinetoHorizontalAbs(SVGPathSegLinetoHorizontalAbsImpl *other) : SVGPathSeg(other)
{
	impl = other;
}

SVGPathSegLinetoHorizontalAbs::~SVGPathSegLinetoHorizontalAbs()
{
	delete impl;
}
   
void SVGPathSegLinetoHorizontalAbs::setX(const float &x)
{
	if(impl)
		impl->setX(x);
}
	    
float SVGPathSegLinetoHorizontalAbs::x() const
{
	if(!impl) return -1;
	return impl->x();
}





SVGPathSegLinetoHorizontalRel::SVGPathSegLinetoHorizontalRel() : SVGPathSeg()
{
	impl = new SVGPathSegLinetoHorizontalRelImpl();
}
 
SVGPathSegLinetoHorizontalRel::SVGPathSegLinetoHorizontalRel(const SVGPathSegLinetoHorizontalRel &other) : SVGPathSeg(other)
{
	impl = other.impl;
}

SVGPathSegLinetoHorizontalRel::SVGPathSegLinetoHorizontalRel(SVGPathSegLinetoHorizontalRelImpl *other) : SVGPathSeg(other)
{
	impl = other;
}

SVGPathSegLinetoHorizontalRel::~SVGPathSegLinetoHorizontalRel()
{
	delete impl;
}
   
void SVGPathSegLinetoHorizontalRel::setX(const float &x)
{
	if(impl)
		impl->setX(x);
}
	    
float SVGPathSegLinetoHorizontalRel::x() const
{
	if(!impl) return -1;
	return impl->x();
}

// vim:ts=4:noet
