/* Automatically generated from impl/SVGPathSegArcImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGPathSegArcAbsImpl__s_hashTableStrings[] = {
    "\0"
    "largeArcFlag\0"
    "sweepFlag\0"
    "angle\0"
    "r1\0"
    "r2\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegArcAbsImpl__s_hashTableEntries[] = {
   { 38, SVGPathSegArcAbsImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 24, SVGPathSegArcAbsImpl::Angle, DontDelete, 0, 12 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 30, SVGPathSegArcAbsImpl::R1, DontDelete, 0, -1 },
   { 36, SVGPathSegArcAbsImpl::X, DontDelete, 0, 11 },
   { 33, SVGPathSegArcAbsImpl::R2, DontDelete, 0, -1 },
   { 1, SVGPathSegArcAbsImpl::LargeArcFlag, DontDelete, 0, 13 },
   { 14, SVGPathSegArcAbsImpl::SweepFlag, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegArcAbsImpl::s_hashTable = { 2, 14, SVGPathSegArcAbsImpl__s_hashTableEntries, 11, SVGPathSegArcAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGPathSegArcRelImpl__s_hashTableStrings[] = {
    "\0"
    "largeArcFlag\0"
    "sweepFlag\0"
    "angle\0"
    "r1\0"
    "r2\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegArcRelImpl__s_hashTableEntries[] = {
   { 38, SVGPathSegArcRelImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 24, SVGPathSegArcRelImpl::Angle, DontDelete, 0, 12 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 30, SVGPathSegArcRelImpl::R1, DontDelete, 0, -1 },
   { 36, SVGPathSegArcRelImpl::X, DontDelete, 0, 11 },
   { 33, SVGPathSegArcRelImpl::R2, DontDelete, 0, -1 },
   { 1, SVGPathSegArcRelImpl::LargeArcFlag, DontDelete, 0, 13 },
   { 14, SVGPathSegArcRelImpl::SweepFlag, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegArcRelImpl::s_hashTable = { 2, 14, SVGPathSegArcRelImpl__s_hashTableEntries, 11, SVGPathSegArcRelImpl__s_hashTableStrings};

} // namespace
