/* Automatically generated from impl/SVGFitToViewBoxImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGFitToViewBoxImpl__s_hashTableStrings[] = {
    "\0"
    "preserveAspectRatio\0"
    "viewBox\0"
};


static const struct HashEntry SVGFitToViewBoxImpl__s_hashTableEntries[] = {
   { 1, SVGFitToViewBoxImpl::PreserveAspectRatio, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 21, SVGFitToViewBoxImpl::ViewBox, DontDelete, 0, -1 }
};

const struct HashTable SVGFitToViewBoxImpl::s_hashTable = { 2, 3, SVGFitToViewBoxImpl__s_hashTableEntries, 3, SVGFitToViewBoxImpl__s_hashTableStrings};

} // namespace
