/* Automatically generated from impl/SVGPathSegCurvetoCubicSmoothImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGPathSegCurvetoCubicSmoothAbsImpl__s_hashTableStrings[] = {
    "\0"
    "x2\0"
    "y2\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegCurvetoCubicSmoothAbsImpl__s_hashTableEntries[] = {
   { 7, SVGPathSegCurvetoCubicSmoothAbsImpl::X, DontDelete, 0, 5 },
   { 9, SVGPathSegCurvetoCubicSmoothAbsImpl::Y, DontDelete, 0, 6 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegCurvetoCubicSmoothAbsImpl::X2, DontDelete, 0, -1 },
   { 4, SVGPathSegCurvetoCubicSmoothAbsImpl::Y2, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegCurvetoCubicSmoothAbsImpl::s_hashTable = { 2, 7, SVGPathSegCurvetoCubicSmoothAbsImpl__s_hashTableEntries, 5, SVGPathSegCurvetoCubicSmoothAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGPathSegCurvetoCubicSmoothRelImpl__s_hashTableStrings[] = {
    "\0"
    "x2\0"
    "y2\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegCurvetoCubicSmoothRelImpl__s_hashTableEntries[] = {
   { 7, SVGPathSegCurvetoCubicSmoothRelImpl::X, DontDelete, 0, 5 },
   { 9, SVGPathSegCurvetoCubicSmoothRelImpl::Y, DontDelete, 0, 6 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegCurvetoCubicSmoothRelImpl::X2, DontDelete, 0, -1 },
   { 4, SVGPathSegCurvetoCubicSmoothRelImpl::Y2, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegCurvetoCubicSmoothRelImpl::s_hashTable = { 2, 7, SVGPathSegCurvetoCubicSmoothRelImpl__s_hashTableEntries, 5, SVGPathSegCurvetoCubicSmoothRelImpl__s_hashTableStrings};

} // namespace
