/* Automatically generated from impl/SVGRectElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGRectElementImpl__s_hashTableStrings[] = {
    "\0"
    "height\0"
    "width\0"
    "rx\0"
    "ry\0"
};


static const struct HashEntry SVGRectElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 15, SVGRectElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 18, SVGRectElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGRectElementImpl::Height, DontDelete|ReadOnly, 0, 7 },
   { 17, SVGRectElementImpl::Ry, DontDelete|ReadOnly, 0, -1 },
   { 8, SVGRectElementImpl::Width, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 14, SVGRectElementImpl::Rx, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGRectElementImpl::s_hashTable = { 2, 8, SVGRectElementImpl__s_hashTableEntries, 7, SVGRectElementImpl__s_hashTableStrings};

} // namespace
