<!DOCTYPE kpartgui>
<kpartplugin name="kvieweffects" library="kview_effectsplugin" version="2">
	<MenuBar>
		<Menu name="effects"><Text>Effe&amp;cts</Text>
			<Action name="plugin_effects_gamma"/>
			<Action name="plugin_effects_blend"/>
			<Action name="plugin_effects_intensity"/>
		</Menu>
	</MenuBar>
</kpartplugin>
