/* Automatically generated from impl/SVGLangSpaceImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGLangSpaceImpl__s_hashTableStrings[] = {
    "\0"
    "xmlspace\0"
    "xmllang\0"
};


static const struct HashEntry SVGLangSpaceImpl__s_hashTableEntries[] = {
   { 10, SVGLangSpaceImpl::XmlLang, DontDelete, 0, -1 },
   { 1, SVGLangSpaceImpl::XmlSpace, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 13, SVGLangSpaceImpl::XmlLang, DontDelete, 0, -1 },
   { 4, SVGLangSpaceImpl::XmlSpace, DontDelete, 0, -1 }
};

const struct HashTable SVGLangSpaceImpl::s_hashTable = { 2, 5, SVGLangSpaceImpl__s_hashTableEntries, 5, SVGLangSpaceImpl__s_hashTableStrings};

} // namespace
