<!DOCTYPE kpartgui>
<kpartplugin name="kviewpresenter" library="kview_presenterplugin" version="3">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<Action name="plugin_presenter_openFiles" group="open_merge_group"/>
		</Menu>
		<Menu name="view"><text>&amp;View</text>
		</Menu>

		<Menu name="go_document"><Text>&amp;Go</Text>
			<Action name="plugin_presenter_imageList"/>
			<Action name="plugin_presenter_slideshow"/>
			<Action name="plugin_presenter_prev"/>
			<Action name="plugin_presenter_next"/>
		</Menu>
	</MenuBar>
	<Menu name="popupmenu">
		<Action name="plugin_presenter_slideshow"/>
	</Menu>
</kpartplugin>
