/*  This file is part of the KDE project
    Copyright (C) 2003 Matthias Kretz <kretz@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2
    as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA  02111-1307, USA.

*/
// $Id: kviewvieweriface.h 243629 2003-08-19 20:00:43Z mkretz $

#ifndef KVIEWVIEWERIFACE_H
#define KVIEWVIEWERIFACE_H

class KViewViewerIface
{
};

#endif // KVIEWVIEWERIFACE_H

// vim: sw=4 ts=4
