/* Automatically generated from impl/SVGPaintImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGPaintImpl__s_hashTableStrings[] = {
    "\0"
    "paintType\0"
    "uri\0"
};


static const struct HashEntry SVGPaintImpl__s_hashTableEntries[] = {
   { 11, SVGPaintImpl::URI, DontDelete, 0, -1 },
   { 1, SVGPaintImpl::PaintType, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPaintImpl::s_hashTable = { 2, 3, SVGPaintImpl__s_hashTableEntries, 3, SVGPaintImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGPaintImplConstructor__s_hashTableStrings[] = {
    "\0"
    "SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR\0"
    "SVG_PAINTTYPE_URI_CURRENTCOLOR\0"
    "SVG_PAINTTYPE_CURRENTCOLOR\0"
    "SVG_PAINTTYPE_URI_RGBCOLOR\0"
    "SVG_PAINTTYPE_RGBCOLOR\0"
    "SVG_PAINTTYPE_URI_NONE\0"
    "SVG_PAINTTYPE_UNKNOWN\0"
    "SVG_PAINTTYPE_NONE\0"
    "SVG_PAINTTYPE_URI\0"
};


static const struct HashEntry SVGPaintImplConstructor__s_hashTableEntries[] = {
   { 145, KSVG::SVG_PAINTTYPE_URI_NONE, DontDelete|ReadOnly, 0, -1 },
   { 95, KSVG::SVG_PAINTTYPE_URI_RGBCOLOR, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, KSVG::SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR, DontDelete|ReadOnly, 0, -1 },
   { 68, KSVG::SVG_PAINTTYPE_CURRENTCOLOR, DontDelete|ReadOnly, 0, -1 },
   { 190, KSVG::SVG_PAINTTYPE_NONE, DontDelete|ReadOnly, 0, -1 },
   { 122, KSVG::SVG_PAINTTYPE_RGBCOLOR, DontDelete|ReadOnly, 0, 11 },
   { 209, KSVG::SVG_PAINTTYPE_URI, DontDelete|ReadOnly, 0, -1 },
   { 168, KSVG::SVG_PAINTTYPE_UNKNOWN, DontDelete|ReadOnly, 0, -1 },
   { 37, KSVG::SVG_PAINTTYPE_URI_CURRENTCOLOR, DontDelete|ReadOnly, 0, -1 },
   { 122, KSVG::SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGPaintImplConstructor::s_hashTable = { 2, 12, SVGPaintImplConstructor__s_hashTableEntries, 11, SVGPaintImplConstructor__s_hashTableStrings};

} // namespace
