/* Automatically generated from impl/SVGUseElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGUseElementImpl__s_hashTableStrings[] = {
    "\0"
    "animatedInstanceRoot\0"
    "instanceRoot\0"
    "height\0"
    "width\0"
    "href\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGUseElementImpl__s_hashTableEntries[] = {
   { 55, SVGUseElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 48, SVGUseElementImpl::Href, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 42, SVGUseElementImpl::Width, DontDelete|ReadOnly, 0, -1 },
   { 35, SVGUseElementImpl::Height, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 22, SVGUseElementImpl::InstanceRoot, DontDelete|ReadOnly, 0, 11 },
   { 0, 0, 0, 0, -1 },
   { 53, SVGUseElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGUseElementImpl::AnimatedInstanceRoot, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGUseElementImpl::s_hashTable = { 2, 12, SVGUseElementImpl__s_hashTableEntries, 11, SVGUseElementImpl__s_hashTableStrings};

} // namespace
