/* Automatically generated from impl/SVGAnimatedPointsImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGAnimatedPointsImpl__s_hashTableStrings[] = {
    "\0"
    "animatedPoints\0"
    "points\0"
};


static const struct HashEntry SVGAnimatedPointsImpl__s_hashTableEntries[] = {
   { 16, SVGAnimatedPointsImpl::Points, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGAnimatedPointsImpl::AnimatedPoints, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGAnimatedPointsImpl::s_hashTable = { 2, 3, SVGAnimatedPointsImpl__s_hashTableEntries, 3, SVGAnimatedPointsImpl__s_hashTableStrings};

} // namespace
