<!DOCTYPE kpartgui>
<kpartgui name="KGhostView" version="22">
<State name="initState">
    <Disable>
        <Action name="file_save_as" />
        <Action name="info" />
        <Action name="orientation_menu" />
        <Action name="media_menu" />
        <Action name="zoomOut" />
        <Action name="zoomTo" />
        <Action name="zoomIn" />
	<Action name="fit_to_page" />
	<Action name="fit_to_screen" />
        <Action name="prevPage" />
        <Action name="nextPage" />
        <Action name="goToStart" />
        <Action name="goToEnd" />
        <Action name="goToPage" />
        <Action name="readUp" />
        <Action name="readDown" />
        <Action name="mark_current" />
        <Action name="mark_all" />
        <Action name="mark_even" />
        <Action name="mark_odd" />
        <Action name="toggle" />
        <Action name="remove" />
        <Action name="no_flicker" />
    </Disable>
</State>
<State name="documentState">
    <Enable>
        <Action name="file_save_as" />
        <Action name="info" />
        <Action name="orientation_menu" />
        <Action name="media_menu" />
        <Action name="zoomOut" />
	<Action name="zoomTo" />
        <Action name="zoomIn" />
	<Action name="fit_to_page" />
	<Action name="fit_to_screen" />
        <Action name="prevPage" />
        <Action name="nextPage" />
        <Action name="goToStart" />
        <Action name="goToEnd" />
        <Action name="goToPage" />
        <Action name="readUp" />
        <Action name="readDown" />
        <Action name="mark_current" />
        <Action name="mark_all" />
        <Action name="mark_even" />
        <Action name="mark_odd" />
        <Action name="toggle" />
        <Action name="remove" />
        <Action name="no_flicker" />
    </Enable>
</State>
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_save_as" group="file_save" />
    <Action name="info" group="file_print" />
  </Menu>
  <Menu name="edit"><text>&amp;Edit</text>
    <Action name="mark_current"/>
    <Action name="mark_all"/>
    <Action name="mark_even"/>
    <Action name="mark_odd"/>
    <Action name="toggle"/>
    <Action name="remove"/>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Action name="orientation_menu"/>
    <Action name="media_menu"/>
    <Separator/>
    <Action name="zoomIn"/>
    <Action name="zoomTo" />
    <Action name="zoomOut"/>
    <Action name="fit_to_page"/>
    <Action name="fit_to_screen" />
    <Action name="prevPage"/>
    <Action name="nextPage"/>
    <Action name="goToStart"/>
    <Action name="goToEnd"/>
    <Action name="goToPage"/>
    <Action name="readUp"/>
    <Action name="readDown"/>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="show_scrollbars"/>
    <Action name="show_page_list"/>
    <Action name="show_page_labels"/>
    <Separator/>
    <Action name="watch_file"/>
    <Action name="no_flicker" />
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure" group="options_configure"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <text>&amp;Main Toolbar</text>
  <Action name="zoomIn"/>
  <Action name="zoomTo" />
  <Action name="zoomOut"/>
  <Action name="goToStart"/>
  <Action name="readUp"/>
  <Action name="readDown"/>
  <Action name="goToEnd"/>
</ToolBar>
</kpartgui>

