/* Automatically generated from impl/SVGViewElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGViewElementImpl__s_hashTableStrings[] = {
    "\0"
    "viewTarget\0"
};


static const struct HashEntry SVGViewElementImpl__s_hashTableEntries[] = {
   { 1, SVGViewElementImpl::ViewTarget, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGViewElementImpl::s_hashTable = { 2, 2, SVGViewElementImpl__s_hashTableEntries, 2, SVGViewElementImpl__s_hashTableStrings};

} // namespace
