/* Automatically generated from impl/SVGAnimatedPathDataImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGAnimatedPathDataImpl__s_hashTableStrings[] = {
    "\0"
    "animatedNormalizedPathSegList\0"
    "normalizedPathSegList\0"
    "animatedPathSegList\0"
    "pathSegList\0"
};


static const struct HashEntry SVGAnimatedPathDataImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 53, SVGAnimatedPathDataImpl::AnimatedPathSegList, DontDelete|ReadOnly, 0, 6 },
   { 0, 0, 0, 0, -1 },
   { 73, SVGAnimatedPathDataImpl::PathSegList, DontDelete|ReadOnly, 0, 5 },
   { 0, 0, 0, 0, -1 },
   { 31, SVGAnimatedPathDataImpl::NormalizedPathSegList, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGAnimatedPathDataImpl::AnimatedNormalizedPathSegList, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGAnimatedPathDataImpl::s_hashTable = { 2, 7, SVGAnimatedPathDataImpl__s_hashTableEntries, 5, SVGAnimatedPathDataImpl__s_hashTableStrings};

} // namespace
