/* Automatically generated from impl/SVGICCColorImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGICCColorImpl__s_hashTableStrings[] = {
    "\0"
    "colorProfile\0"
    "colors\0"
};


static const struct HashEntry SVGICCColorImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGICCColorImpl::ColorProfile, DontDelete, 0, 3 },
   { 0, 0, 0, 0, -1 },
   { 14, SVGICCColorImpl::Colors, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGICCColorImpl::s_hashTable = { 2, 4, SVGICCColorImpl__s_hashTableEntries, 3, SVGICCColorImpl__s_hashTableStrings};

} // namespace
