/* Automatically generated from impl/SVGElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGElementImpl__s_hashTableStrings[] = {
    "\0"
    "ownerSVGElement\0"
    "viewportElement\0"
    "onmouseclick\0"
    "onmousedown\0"
    "onmousemove\0"
    "onmouseover\0"
    "onactivate\0"
    "onfocusout\0"
    "onkeypress\0"
    "onmouseout\0"
    "onfocusin\0"
    "onkeydown\0"
    "onmouseup\0"
    "onabort\0"
    "onclick\0"
    "onerror\0"
    "onkeyup\0"
    "xmlbase\0"
    "onload\0"
    "id\0"
};


static const struct HashEntry SVGElementImpl__s_hashTableEntries[] = {
   { 17, SVGElementImpl::ViewportElement, DontDelete|ReadOnly, 0, -1 },
   { 136, SVGElementImpl::OnKeyDown, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 164, SVGElementImpl::OnClick, DontDelete, 0, 27 },
   { 33, SVGElementImpl::OnClick, DontDelete, 0, -1 },
   { 93, SVGElementImpl::OnFocusOut, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 188, SVGElementImpl::XmlBase, DontDelete, 0, 25 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGElementImpl::OwnerSvgElement, DontDelete|ReadOnly, 0, 23 },
   { 0, 0, 0, 0, -1 },
   { 196, SVGElementImpl::OnLoad, DontDelete, 0, 29 },
   { 58, SVGElementImpl::OnMouseMove, DontDelete, 0, -1 },
   { 46, SVGElementImpl::OnMouseDown, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 191, SVGElementImpl::XmlBase, DontDelete, 0, 26 },
   { 203, SVGElementImpl::ElementId, DontDelete, 0, 30 },
   { 70, SVGElementImpl::OnMouseOver, DontDelete, 0, -1 },
   { 146, SVGElementImpl::OnMouseUp, DontDelete, 0, 24 },
   { 115, SVGElementImpl::OnMouseOut, DontDelete, 0, 28 },
   { 82, SVGElementImpl::OnActivate, DontDelete, 0, -1 },
   { 180, SVGElementImpl::OnKeyUp, DontDelete, 0, -1 },
   { 104, SVGElementImpl::OnKeyPress, DontDelete, 0, -1 },
   { 126, SVGElementImpl::OnFocusIn, DontDelete, 0, -1 },
   { 172, SVGElementImpl::OnError, DontDelete, 0, -1 },
   { 156, SVGElementImpl::OnAbort, DontDelete, 0, -1 }
};

const struct HashTable SVGElementImpl::s_hashTable = { 2, 31, SVGElementImpl__s_hashTableEntries, 23, SVGElementImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGElementImplProto__s_hashTableStrings[] = {
    "\0"
    "getPropertyValue\0"
    "setProperty\0"
    "getStyle\0"
};


static const struct HashEntry SVGElementImplProto__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 18, SVGElementImpl::SetProperty, DontDelete|Function, 2, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGElementImpl::GetPropertyValue, DontDelete|Function, 1, -1 },
   { 30, SVGElementImpl::GetStyle, DontDelete|Function, 0, -1 }
};

const struct HashTable SVGElementImplProto::s_hashTable = { 2, 5, SVGElementImplProto__s_hashTableEntries, 5, SVGElementImplProto__s_hashTableStrings};

} // namespace
