/* Automatically generated from impl/SVGTextPathElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGTextPathElementImpl__s_hashTableStrings[] = {
    "\0"
    "startOffset\0"
    "spacing\0"
    "method\0"
};


static const struct HashEntry SVGTextPathElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 21, SVGTextPathElementImpl::Method, DontDelete|ReadOnly, 0, 5 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGTextPathElementImpl::StartOffset, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 13, SVGTextPathElementImpl::Spacing, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGTextPathElementImpl::s_hashTable = { 2, 6, SVGTextPathElementImpl__s_hashTableEntries, 5, SVGTextPathElementImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGTextPathElementImplConstructor__s_hashTableStrings[] = {
    "\0"
    "TEXTPATH_SPACINGTYPE_UNKNOWN\0"
    "TEXTPATH_METHODTYPE_STRETCH\0"
    "TEXTPATH_METHODTYPE_UNKNOWN\0"
    "TEXTPATH_SPACINGTYPE_EXACT\0"
    "TEXTPATH_METHODTYPE_ALIGN\0"
    "TEXTPATH_SPACINGTYPE_AUTO\0"
};


static const struct HashEntry SVGTextPathElementImplConstructor__s_hashTableEntries[] = {
   { 30, KSVG::TEXTPATH_METHODTYPE_STRETCH, DontDelete|ReadOnly, 0, -1 },
   { 139, KSVG::TEXTPATH_SPACINGTYPE_AUTO, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, KSVG::TEXTPATH_SPACINGTYPE_UNKNOWN, DontDelete|ReadOnly, 0, -1 },
   { 113, KSVG::TEXTPATH_METHODTYPE_ALIGN, DontDelete|ReadOnly, 0, -1 },
   { 58, KSVG::TEXTPATH_METHODTYPE_UNKNOWN, DontDelete|ReadOnly, 0, 7 },
   { 0, 0, 0, 0, -1 },
   { 86, KSVG::TEXTPATH_SPACINGTYPE_EXACT, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGTextPathElementImplConstructor::s_hashTable = { 2, 8, SVGTextPathElementImplConstructor__s_hashTableEntries, 7, SVGTextPathElementImplConstructor__s_hashTableStrings};

} // namespace
