/* Automatically generated from impl/SVGRadialGradientElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGRadialGradientElementImpl__s_hashTableStrings[] = {
    "\0"
    "cx\0"
    "cy\0"
    "fx\0"
    "fy\0"
    "r\0"
};


static const struct HashEntry SVGRadialGradientElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGRadialGradientElementImpl::Cx, DontDelete|ReadOnly, 0, 7 },
   { 4, SVGRadialGradientElementImpl::Cy, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 7, SVGRadialGradientElementImpl::Fx, DontDelete|ReadOnly, 0, -1 },
   { 10, SVGRadialGradientElementImpl::Fy, DontDelete|ReadOnly, 0, -1 },
   { 13, SVGRadialGradientElementImpl::R, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGRadialGradientElementImpl::s_hashTable = { 2, 8, SVGRadialGradientElementImpl__s_hashTableEntries, 7, SVGRadialGradientElementImpl__s_hashTableStrings};

} // namespace
