/* Automatically generated from impl/SVGEllipseElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGEllipseElementImpl__s_hashTableStrings[] = {
    "\0"
    "cx\0"
    "cy\0"
    "rx\0"
    "ry\0"
};


static const struct HashEntry SVGEllipseElementImpl__s_hashTableEntries[] = {
   { 4, SVGEllipseElementImpl::Cy, DontDelete|ReadOnly, 0, 6 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGEllipseElementImpl::Cx, DontDelete|ReadOnly, 0, 5 },
   { 7, SVGEllipseElementImpl::Rx, DontDelete|ReadOnly, 0, -1 },
   { 10, SVGEllipseElementImpl::Ry, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGEllipseElementImpl::s_hashTable = { 2, 7, SVGEllipseElementImpl__s_hashTableEntries, 5, SVGEllipseElementImpl__s_hashTableStrings};

} // namespace
