<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KPovModelerPart" version="14">

<MenuBar>
   <Menu name="file"><text>&amp;File</text>
      <Action name="file_export" group="file_export_actions"/>
      <Action name="file_import" group="file_import_actions"/>
   </Menu>
   <Menu name="edit"><text>&amp;Edit</text>
      <Action name="edit_undo"/>
      <Action name="edit_redo"/>
      <Separator/>
      <Action name="edit_cut"/>
      <Action name="edit_copy"/>
      <Action name="edit_paste"/>
      <Action name="edit_delete"/>
   </Menu>
   <Menu name="insert"><text>&amp;Insert</text>
      <Action name="new_globalsettings"/>
      <Action name="new_radiosity"/>
      <Action name="new_globalphotons"/>
      <Separator/>
      <Action name="new_declare"/>
      <Separator/>
      <Action name="new_clippedby"/>
      <Action name="new_boundedby"/>
      <Separator/>
      <Action name="new_light"/>
      <Action name="new_lightgroup"/>
      <Action name="new_lookslike"/>
      <Action name="new_projectedthrough"/>
      <Action name="new_camera"/>
      <Separator/>
      <Action name="new_comment"/>
      <Action name="new_raw"/>
      <Separator/>
      <Menu name="finite_solid_objects"><text>Finite Solid Primitives</text>
        <Action name="new_box"/>
        <Action name="new_sphere"/>
        <Action name="new_cylinder"/>
        <Action name="new_cone"/>
        <Action name="new_torus"/>
        <Action name="new_superquadricellipsoid"/>
        <Action name="new_heightfield"/>
        <Action name="new_text"/>
        <Action name="new_juliafractal"/>
   <Action name="new_isosurface"/>
        <Separator/>
        <Action name="new_blob"/>
        <Action name="new_blobsphere"/>
        <Action name="new_blobcylinder"/>
        <Separator/>
        <Action name="new_lathe"/>
        <Action name="new_prism"/>
        <Action name="new_surfaceofrevolution"/>
   <Action name="new_spheresweep"/>
        <Separator/>
        <Action name="new_objectlink"/>
      </Menu>
      <Menu name="finite_patch_objects"><text>Finite Patch Primitives</text>
        <Action name="new_disc"/>
        <Action name="new_bicubicpatch"/>
        <Action name="new_triangle"/>
        <Action name="new_mesh"/>
      </Menu>
      <Menu name="infinite_solid_objects"><text>Infinite Solid Primitives</text>
   <Action name="new_plane"/>
        <Action name="new_polynom"/>
      </Menu>
      <Menu name="csg_objects"><text>Constructive Solid Geometry</text>
        <Action name="new_union"/>
        <Action name="new_intersection"/>
        <Action name="new_difference"/>
        <Action name="new_merge"/>
      </Menu>
      <Separator/>
      <Menu name="material_objects"><text>Material</text>
        <Action name="new_material"/>
      </Menu>
      <Menu name="interior_objects"><text>Interior</text>
        <Action name="new_interior"/>
        <Action name="new_media"/>
        <Action name="new_density"/>
        <Action name="new_densitylist"/>
        <Action name="new_densitymap"/>
      </Menu>
      <Menu name="texture_objects"><text>Textures</text>
        <Action name="new_pattern"/>
        <Action name="new_blendmapmodifiers"/>
        <Action name="new_warp"/>
        <Separator/>
        <Action name="new_texture"/>
        <Action name="new_interiortexture"/>
        <Action name="new_texturelist"/>
        <Action name="new_texturemap"/>
        <Action name="new_materialmap"/>
        <Action name="new_pigment"/>
        <Action name="new_pigmentlist"/>
        <Action name="new_pigmentmap"/>
        <Action name="new_imagemap"/>
        <Action name="new_solidcolor"/>
        <Action name="new_colorlist"/>
        <Action name="new_colormap"/>
        <Action name="new_normal"/>
        <Action name="new_normallist"/>
        <Action name="new_normalmap"/>
        <Action name="new_bumpmap"/>
        <Action name="new_slope"/>
        <Action name="new_slopemap"/>
        <Action name="new_finish"/>
        <Separator/>
        <Action name="new_quickcolor"/>
      </Menu>
      <Menu name="photons_objects"><text>Photons</text>
        <Action name="new_photons"/>
      </Menu>
      <Menu name="atmospheric_objects"><text>Atmospheric Effects</text>
        <Action name="new_skysphere"/>
        <Action name="new_rainbow"/>
        <Action name="new_fog"/>
      </Menu>
      <Separator/>
      <Menu name="transformation_objects"><text>Transformations</text>
        <Action name="new_scale"/>
        <Action name="new_rotate"/>
        <Action name="new_translate"/>
        <Action name="new_povraymatrix"/>
      </Menu>
   </Menu>
   <Menu name="view"><text>&amp;View</text>
      <Action name="view_render_settings"/>
      <Action name="view_render"/>
      <Action name="view_render_window"/>
      <Separator/>
      <Action name="global_detail_level"/>
   </Menu>
</MenuBar>

<Menu name="viewPopup">
   <Action name="view_redisplay"/>
   <Separator/>
   <Action name="view_pos_x"/>
   <Action name="view_neg_x"/>
   <Action name="view_pos_y"/>
   <Action name="view_neg_y"/>
   <Action name="view_pos_z"/>
   <Action name="view_neg_z"/>
   <Action name="view_cameras_menu"/>
   <Separator/>
   <Action name="view_translate"/>
   <Action name="view_scale"/>
   <Separator/>
   <Action name="cp_snaptogrid"/>
   <ActionList name="cp_commands"/>
   <Separator/>
   <Action name="view_control_points_menu"/>
</Menu>

<Menu name="treeViewPopup">
   <Action name="edit_cut"/>
   <Action name="edit_copy"/>
   <Action name="edit_paste"/>
   <Action name="edit_delete"/>
</Menu>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
   <Action name="edit_undo"/>
   <Action name="edit_redo"/>
   <Separator/>
   <Action name="edit_cut"/>
   <Action name="edit_copy"/>
   <Action name="edit_paste"/>
   <Action name="edit_delete"/>
</ToolBar>

<ToolBar name="newSolidObjectsBar"><text>Finite Solid Primitives</text>
   <Action name="new_box"/>
   <Action name="new_sphere"/>
   <Action name="new_cylinder"/>
   <Action name="new_cone"/>
   <Action name="new_torus"/>
   <Action name="new_superquadricellipsoid"/>
   <Action name="new_heightfield"/>
   <Action name="new_text"/>
   <Action name="new_juliafractal"/>
   <Action name="new_isosurface"/>
   <Separator/>
   <Action name="new_blob"/>
   <Action name="new_blobsphere"/>
   <Action name="new_blobcylinder"/>
   <Separator/>
   <Action name="new_lathe"/>
   <Action name="new_prism"/>
   <Action name="new_surfaceofrevolution"/>
   <Action name="new_spheresweep"/>
   <Separator/>
   <Action name="new_objectlink"/>
</ToolBar>

<ToolBar name="newCSGObjectsBar"><text>Constructive Solid Geometry</text>
   <Action name="new_union"/>
   <Action name="new_intersection"/>
   <Action name="new_difference"/>
   <Action name="new_merge"/>
</ToolBar>

<ToolBar name="newInfiniteAndPatchObjectsBar"><text>Infinite and Patch Primitives</text>
   <Action name="new_plane"/>
   <Action name="new_polynom"/>
   <Separator/>
   <Action name="new_disc"/>
   <Action name="new_bicubicpatch"/>
   <Action name="new_triangle"/>
   <Action name="new_mesh"/>
</ToolBar>

<ToolBar name="newAtmosphericEffectsBar"><text>Atmospheric Effects</text>
   <Action name="new_skysphere"/>
   <Action name="new_rainbow"/>
   <Action name="new_fog"/>
</ToolBar>

<ToolBar name="newMiscObjectsBar"><text>Misc Objects</text>
   <Action name="new_declare"/>
   <Separator/>
   <Action name="new_clippedby"/>
   <Action name="new_boundedby"/>
   <Separator/>
   <Action name="new_light"/>
   <Action name="new_lightgroup"/>
   <Action name="new_lookslike"/>
   <Action name="new_projectedthrough"/>
   <Separator/>
   <Action name="new_camera"/>
   <Separator/>
   <Action name="new_globalsettings"/>
   <Action name="new_radiosity"/>
   <Action name="new_globalphotons"/>
   <Separator/>
   <Action name="new_comment"/>
   <Action name="new_raw"/>
</ToolBar>

<ToolBar name="newTransformationsBar"><text>Transformations</text>
   <Action name="new_scale"/>
   <Action name="new_rotate"/>
   <Action name="new_translate"/>
   <Action name="new_povraymatrix"/>
</ToolBar>

<ToolBar name="newTexturesBar"><text>Textures</text>
   <Action name="new_material"/>
   <Separator/>
   <Action name="new_interior"/>
   <Action name="new_media"/>
   <Action name="new_density"/>
   <Action name="new_densitylist"/>
   <Action name="new_densitymap"/>
   <Separator/>
   <Action name="new_pattern"/>
   <Action name="new_blendmapmodifiers"/>
   <Action name="new_warp"/>
   <Separator/>
   <Action name="new_texture"/>
   <Action name="new_interiortexture"/>
   <Action name="new_texturelist"/>
   <Action name="new_texturemap"/>
   <Action name="new_materialmap"/>
   <Action name="new_pigment"/>
   <Action name="new_pigmentlist"/>
   <Action name="new_pigmentmap"/>
   <Action name="new_imagemap"/>
   <Action name="new_solidcolor"/>
   <Action name="new_colorlist"/>
   <Action name="new_colormap"/>
   <Action name="new_normal"/>
   <Action name="new_normallist"/>
   <Action name="new_normalmap"/>
   <Action name="new_bumpmap"/>
   <Action name="new_slope"/>
   <Action name="new_slopemap"/>
   <Action name="new_finish"/>
   <Separator/>
   <Action name="new_quickcolor"/>
   <Separator/>
   <Action name="new_photons"/>
</ToolBar>

<ToolBar name="renderBar"><text>Povray Rendering</text>
   <Action name="view_render_combo"/>
   <Action name="view_render_settings"/>
   <Action name="view_render"/>
</ToolBar>

<ToolBar name="graphicalViewsBar"><text>Graphical View</text>
   <Action name="view_visibility_label"/>
   <Action name="view_visibility_level"/>
   <Separator/>
   <Action name="global_detail_label"/>
   <Action name="global_detail_level"/>
</ToolBar>

</kpartgui>
