/* Automatically generated from impl/SVGStylableImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGStylableImpl__s_hashTableStrings[] = {
    "\0"
    "glyph-orientation-horizontal\0"
    "glyph-orientation-vertical\0"
    "color-interpolation\0"
    "stroke-dashoffset\0"
    "stroke-miterlimit\0"
    "stroke-dasharray\0"
    "stroke-linejoin\0"
    "text-decoration\0"
    "baseline-shift\0"
    "letter-spacing\0"
    "pointer-events\0"
    "stroke-linecap\0"
    "stroke-opacity\0"
    "color-profile\0"
    "fill-opacity\0"
    "marker-start\0"
    "stroke-width\0"
    "unicode-bidi\0"
    "word-spacing\0"
    "writing-mode\0"
    "font-family\0"
    "font-weight\0"
    "text-anchor\0"
    "font-style\0"
    "marker-end\0"
    "marker-mid\0"
    "stop-color\0"
    "visibility\0"
    "className\0"
    "clip-path\0"
    "clip-rule\0"
    "direction\0"
    "fill-rule\0"
    "font-size\0"
    "overflow\0"
    "display\0"
    "cursor\0"
    "marker\0"
    "stroke\0"
    "clip\0"
    "fill\0"
    "mask\0"
};


static const struct HashEntry SVGStylableImpl__s_hashTableEntries[] = {
   { 523, SVGStylableImpl::Fill, DontDelete|ReadOnly, 0, -1 },
   { 518, SVGStylableImpl::Clip, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 341, SVGStylableImpl::FontWeight, DontDelete|ReadOnly, 0, -1 },
   { 353, SVGStylableImpl::TextAnchor, DontDelete|ReadOnly, 0, 52 },
   { 511, SVGStylableImpl::Stroke, DontDelete|ReadOnly, 0, 56 },
   { 251, SVGStylableImpl::FillOpacity, DontDelete|ReadOnly, 0, -1 },
   { 398, SVGStylableImpl::StopColor, DontDelete|ReadOnly, 0, 61 },
   { 256, SVGStylableImpl::Opacity, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 95, SVGStylableImpl::StrokeMiterlimit, DontDelete|ReadOnly, 0, 49 },
   { 497, SVGStylableImpl::Cursor, DontDelete|ReadOnly, 0, 60 },
   { 222, SVGStylableImpl::StrokeOpacity, DontDelete|ReadOnly, 0, 62 },
   { 130, SVGStylableImpl::StrokeLineJoin, DontDelete|ReadOnly, 0, 53 },
   { 409, SVGStylableImpl::Visibility, DontDelete|ReadOnly, 0, 57 },
   { 316, SVGStylableImpl::WritingMode, DontDelete|ReadOnly, 0, 58 },
   { 0, 0, 0, 0, -1 },
   { 192, SVGStylableImpl::PointerEvents, DontDelete|ReadOnly, 0, 63 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 450, SVGStylableImpl::Direction, DontDelete|ReadOnly, 0, 51 },
   { 0, 0, 0, 0, -1 },
   { 264, SVGStylableImpl::MarkerStart, DontDelete|ReadOnly, 0, 59 },
   { 0, 0, 0, 0, -1 },
   { 237, SVGStylableImpl::ColorProfile, DontDelete|ReadOnly, 0, -1 },
   { 420, SVGStylableImpl::ClassName, DontDelete|ReadOnly, 0, 48 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 57, SVGStylableImpl::ColorInterpolation, DontDelete|ReadOnly, 0, -1 },
   { 277, SVGStylableImpl::StrokeWidth, DontDelete|ReadOnly, 0, 47 },
   { 177, SVGStylableImpl::LetterSpacing, DontDelete|ReadOnly, 0, -1 },
   { 77, SVGStylableImpl::StrokeDashOffset, DontDelete|ReadOnly, 0, -1 },
   { 470, SVGStylableImpl::FontSize, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 480, SVGStylableImpl::Overflow, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 370, SVGStylableImpl::Style, DontDelete|ReadOnly, 0, -1 },
   { 329, SVGStylableImpl::FontFamily, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 207, SVGStylableImpl::StrokeLineCap, DontDelete|ReadOnly, 0, 55 },
   { 403, SVGStylableImpl::Color, DontDelete|ReadOnly, 0, -1 },
   { 365, SVGStylableImpl::FontStyle, DontDelete|ReadOnly, 0, 50 },
   { 146, SVGStylableImpl::TextDecoration, DontDelete|ReadOnly, 0, 54 },
   { 290, SVGStylableImpl::UnicodeBidi, DontDelete|ReadOnly, 0, -1 },
   { 430, SVGStylableImpl::ClipPath, DontDelete|ReadOnly, 0, 64 },
   { 387, SVGStylableImpl::MarkerMid, DontDelete|ReadOnly, 0, -1 },
   { 376, SVGStylableImpl::MarkerEnd, DontDelete|ReadOnly, 0, -1 },
   { 504, SVGStylableImpl::Marker, DontDelete|ReadOnly, 0, -1 },
   { 489, SVGStylableImpl::Display, DontDelete|ReadOnly, 0, -1 },
   { 460, SVGStylableImpl::FillRule, DontDelete|ReadOnly, 0, -1 },
   { 440, SVGStylableImpl::ClipRule, DontDelete|ReadOnly, 0, -1 },
   { 113, SVGStylableImpl::StrokeDashArray, DontDelete|ReadOnly, 0, -1 },
   { 162, SVGStylableImpl::BaselineShift, DontDelete|ReadOnly, 0, -1 },
   { 303, SVGStylableImpl::WordSpacing, DontDelete|ReadOnly, 0, -1 },
   { 30, SVGStylableImpl::GlyphOrientationVertical, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGStylableImpl::GlyphOrientationHorizontal, DontDelete|ReadOnly, 0, -1 },
   { 528, SVGStylableImpl::Mask, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGStylableImpl::s_hashTable = { 2, 65, SVGStylableImpl__s_hashTableEntries, 47, SVGStylableImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGStylableImplProto__s_hashTableStrings[] = {
    "\0"
    "getStyle\0"
};


static const struct HashEntry SVGStylableImplProto__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGStylableImpl::GetStyle, DontDelete|Function, 0, -1 }
};

const struct HashTable SVGStylableImplProto::s_hashTable = { 2, 2, SVGStylableImplProto__s_hashTableEntries, 2, SVGStylableImplProto__s_hashTableStrings};

} // namespace
