<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KSVGPlugin" version="2">
<MenuBar>
	<Menu name="settings"><text>&amp;Settings</text>
		<Separator/>
		<Action name="view_zoom_in" />
		<Action name="view_zoom_out" />
		<Action name="stop_anims"/>
		<Action name="view_source"/>
		<Action name="font_kerning"/>
		<Action name="rendering_backend"/>
		<Action name="progressive"/>
	</Menu>
</MenuBar>
<Toolbar name="ksvgToolBar">
	<Action name="view_zoom_in" />
	<Action name="view_zoom_out" />
	<Action name="stop_anims"/>
	<Action name="view_source"/>
	<Action name="view_memory"/>
	<Action name="font_kerning"/>
</Toolbar>
<Menu name="popupmenu">
	<Action name="view_zoom_in" />
	<Action name="view_zoom_out" />
	<Action name="zoom_reset" />
	<Action name="stop_anims"/>
	<Action name="view_source"/>
	<Separator weakSeparator="1" />
	<Action name="view_memory"/>
	<Action name="save_to_png"/>
	<Separator weakSeparator="1" />
	<Action name="help_about_app"/>
</Menu>
</kpartgui>
