/* Automatically generated from impl/SVGColorProfileElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGColorProfileElementImpl__s_hashTableStrings[] = {
    "\0"
    "rendering-intent\0"
    "href\0"
    "name\0"
};


static const struct HashEntry SVGColorProfileElementImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 18, SVGColorProfileElementImpl::Href, DontDelete|ReadOnly, 0, 5 },
   { 23, SVGColorProfileElementImpl::Name, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGColorProfileElementImpl::RenderingIntent, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGColorProfileElementImpl::s_hashTable = { 2, 6, SVGColorProfileElementImpl__s_hashTableEntries, 5, SVGColorProfileElementImpl__s_hashTableStrings};

} // namespace
