<!DOCTYPE kpartgui>
<kpartplugin name="kviewbrowser" library="kview_browserplugin">
	<MenuBar>
		<Menu name="tools"><Text>&amp;Tools</Text>
			<Action name="previous_image"/>
			<Action name="next_image"/>
		</Menu>
	</MenuBar>
	<ToolBar name="extraToolBar">
		<text>Extra Toolbar</text>
		<Action name="previous_image"/>
		<Action name="next_image"/>
	</ToolBar>
	<Menu name="popupmenu">
		<Action name="previous_image"/>
		<Action name="next_image"/>
	</Menu>
</kpartplugin>
