//
// Class: documentPagePixmap
//
// Widget for displaying TeX DVI files.
// Part of KDVI- A previewer for TeX DVI files.
//
// (C) 2004 Stefan Kebekus. Distributed under the GPL.


#ifndef _documentpagepixmap_h_
#define _documentpagepixmap_h_

#include "documentPage.h"



// This class contains everything documentRenderer needs to know 
// about a certain page.
class documentPagePixmap: public documentPage, public QPixmap
{
  /** Returns a pointer to the paintDevice (in most implementation
      either a QPixmap, or a QPrinter). The pointer returned is valid
      as long as *this documentPage exists. This method is used by the
      renderer to draw on the page. */
  virtual QPaintDevice *getPaintDevice() {return this;};
};


#endif
