/*  This file is part of the KDE project
    Copyright (C) 2002 Matthias Kretz <kretz@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

*/
// $Id: viewer.cpp 239366 2003-07-27 00:25:16Z mueller $

#include "viewer.h"

namespace KImageViewer
{
	Viewer::Viewer( QObject * parent, const char * name )
		: KParts::ReadWritePart( parent, name )
	{
	}

	Viewer::~Viewer()
	{
	}
} //namespace

#include "viewer.moc"
// vim: sw=4 ts=4
