/* Automatically generated from impl/SVGPathSegMovetoImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGPathSegMovetoAbsImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegMovetoAbsImpl__s_hashTableEntries[] = {
   { 1, SVGPathSegMovetoAbsImpl::X, DontDelete, 0, -1 },
   { 3, SVGPathSegMovetoAbsImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPathSegMovetoAbsImpl::s_hashTable = { 2, 3, SVGPathSegMovetoAbsImpl__s_hashTableEntries, 3, SVGPathSegMovetoAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGPathSegMovetoRelImpl__s_hashTableStrings[] = {
    "\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPathSegMovetoRelImpl__s_hashTableEntries[] = {
   { 1, SVGPathSegMovetoRelImpl::X, DontDelete, 0, -1 },
   { 3, SVGPathSegMovetoRelImpl::Y, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 }
};

const struct HashTable SVGPathSegMovetoRelImpl::s_hashTable = { 2, 3, SVGPathSegMovetoRelImpl__s_hashTableEntries, 3, SVGPathSegMovetoRelImpl__s_hashTableStrings};

} // namespace
