<!DOCTYPE kpartgui>
<kpartgui name="KViewShell" version="2">
<MenuBar>
 <Menu name="file"><text>&amp;File</text>
   <Action name="reload" append="open_merge" />	
   <DefineGroup name="file_print" append="print_merge" />
   <DefineGroup name="file_save" append="save_merge" />
 </Menu>
 <Merge/>
 <Menu name="settings"><text>&amp;Settings</text>
  <DefineGroup name="settings_top" append="show_merge"/>
  <DefineGroup name="configure_merge" append="configure_merge" />
  <Merge/>
 </Menu>
  <Menu name="help"><text>&amp;Help</text>
    <DefineGroup name="help_top"/>
    <DefineGroup name="help_about"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <Action name="reload"/>
</ToolBar>
<StatusBar/>
</kpartgui>

