/* Automatically generated from impl/SVGMaskElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGMaskElementImpl__s_hashTableStrings[] = {
    "\0"
    "maskContentUnits\0"
    "maskUnits\0"
    "height\0"
    "width\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGMaskElementImpl__s_hashTableEntries[] = {
   { 18, SVGMaskElementImpl::MaskUnits, DontDelete|ReadOnly, 0, -1 },
   { 41, SVGMaskElementImpl::X, DontDelete|ReadOnly, 0, -1 },
   { 43, SVGMaskElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 1, SVGMaskElementImpl::MaskContentUnits, DontDelete|ReadOnly, 0, 7 },
   { 0, 0, 0, 0, -1 },
   { 35, SVGMaskElementImpl::Width, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 28, SVGMaskElementImpl::Height, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGMaskElementImpl::s_hashTable = { 2, 8, SVGMaskElementImpl__s_hashTableEntries, 7, SVGMaskElementImpl__s_hashTableStrings};

} // namespace
