/* Automatically generated from ecma/ksvg_window.cpp using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char KSVG__Window__s_hashTableStrings[] = {
    "\0"
    "SVGPreserveAspectRatio\0"
    "SVGTextContentElement\0"
    "getSVGViewerVersion\0"
    "SVGGradientElement\0"
    "SVGTextPathElement\0"
    "SVGMarkerElement\0"
    "SVGZoomAndPan\0"
    "clearInterval\0"
    "SVGTransform\0"
    "clearTimeout\0"
    "innerHeight\0"
    "setInterval\0"
    "svgDocument\0"
    "SVGPathSeg\0"
    "innerWidth\0"
    "setTimeout\0"
    "SVGLength\0"
    "navigator\0"
    "printNode\0"
    "SVGAngle\0"
    "SVGColor\0"
    "SVGPaint\0"
    "document\0"
    "parseXML\0"
    "confirm\0"
    "postURL\0"
    "success\0"
    "closed\0"
    "getURL\0"
    "prompt\0"
    "window\0"
    "alert\0"
    "debug\0"
    "evt\0"
};


static const struct HashEntry KSVG__Window__s_hashTableEntries[] = {
   { 244, KSVG::Window::_SVGLength, DontDelete|ReadOnly, 0, 39 },
   { 0, 0, 0, 0, -1 },
   { 319, KSVG::Window::_Confirm, DontDelete|Function, 1, -1 },
   { 283, KSVG::Window::_SVGColor, DontDelete|ReadOnly, 0, -1 },
   { 162, KSVG::Window::_ClearTimeout, DontDelete|Function, 1, 41 },
   { 0, 0, 0, 0, -1 },
   { 149, KSVG::Window::_SVGTransform, DontDelete|ReadOnly, 0, 38 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 377, KSVG::Window::_Debug, DontDelete|Function, 1, -1 },
   { 0, 0, 0, 0, -1 },
   { 199, KSVG::Window::_Document, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 301, KSVG::Window::_Document, DontDelete|ReadOnly, 0, 34 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 327, KSVG::Window::_PostURL, DontDelete|Function, 5, -1 },
   { 364, KSVG::Window::_Window, DontDelete|ReadOnly, 0, -1 },
   { 175, KSVG::Window::_InnerHeight, DontDelete|ReadOnly, 0, 35 },
   { 0, 0, 0, 0, -1 },
   { 233, KSVG::Window::_SetTimeout, DontDelete|Function, 2, -1 },
   { 343, KSVG::Window::_Closed, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 46, KSVG::Window::_GetSVGViewerVersion, DontDelete|Function, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 371, KSVG::Window::_Alert, DontDelete|Function, 1, -1 },
   { 1, KSVG::Window::_SVGPreserveAspectRatio, DontDelete|ReadOnly, 0, -1 },
   { 357, KSVG::Window::_Prompt, DontDelete|Function, 2, -1 },
   { 383, KSVG::Window::_Evt, DontDelete|ReadOnly, 0, 36 },
   { 135, KSVG::Window::_ClearInterval, DontDelete|Function, 1, -1 },
   { 0, 0, 0, 0, -1 },
   { 222, KSVG::Window::_InnerWidth, DontDelete|ReadOnly, 0, 43 },
   { 121, KSVG::Window::_SVGZoomAndPan, DontDelete|ReadOnly, 0, -1 },
   { 187, KSVG::Window::_SetInterval, DontDelete|Function, 2, 37 },
   { 254, KSVG::Window::_Navigator, DontDelete|ReadOnly, 0, 45 },
   { 264, KSVG::Window::_PrintNode, DontDelete|Function, 1, -1 },
   { 274, KSVG::Window::_SVGAngle, DontDelete|ReadOnly, 0, 44 },
   { 211, KSVG::Window::_SVGPathSeg, DontDelete|ReadOnly, 0, -1 },
   { 66, KSVG::Window::_SVGGradientElement, DontDelete|ReadOnly, 0, 40 },
   { 104, KSVG::Window::_SVGMarkerElement, DontDelete|ReadOnly, 0, 42 },
   { 85, KSVG::Window::_SVGTextPathElement, DontDelete|ReadOnly, 0, -1 },
   { 292, KSVG::Window::_SVGPaint, DontDelete|ReadOnly, 0, -1 },
   { 24, KSVG::Window::_SVGTextContentElement, DontDelete|ReadOnly, 0, 46 },
   { 335, KSVG::Window::_Success, DontDelete|Function, 1, -1 },
   { 350, KSVG::Window::_GetURL, DontDelete|Function, 2, -1 },
   { 310, KSVG::Window::_ParseXML, DontDelete|Function, 1, -1 }
};

const struct HashTable KSVG::Window::s_hashTable = { 2, 47, KSVG__Window__s_hashTableEntries, 34, KSVG__Window__s_hashTableStrings};

} // namespace
