/* Automatically generated from impl/SVGStyleElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGStyleElementImpl__s_hashTableStrings[] = {
    "\0"
    "xmlspace\0"
    "media\0"
    "title\0"
    "type\0"
};


static const struct HashEntry SVGStyleElementImpl__s_hashTableEntries[] = {
   { 22, SVGStyleElementImpl::Type, DontDelete, 0, -1 },
   { 1, SVGStyleElementImpl::Xmlspace, DontDelete, 0, 5 },
   { 10, SVGStyleElementImpl::Media, DontDelete, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 16, SVGStyleElementImpl::Title, DontDelete, 0, -1 }
};

const struct HashTable SVGStyleElementImpl::s_hashTable = { 2, 6, SVGStyleElementImpl__s_hashTableEntries, 5, SVGStyleElementImpl__s_hashTableStrings};

} // namespace
