/* Automatically generated from impl/SVGPathSegLinetoVerticalImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGPathSegLinetoVerticalAbsImpl__s_hashTableStrings[] = {
    "\0"
    "y\0"
};


static const struct HashEntry SVGPathSegLinetoVerticalAbsImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegLinetoVerticalAbsImpl::Y, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegLinetoVerticalAbsImpl::s_hashTable = { 2, 2, SVGPathSegLinetoVerticalAbsImpl__s_hashTableEntries, 2, SVGPathSegLinetoVerticalAbsImpl__s_hashTableStrings};

} // namespace

using namespace KJS;

namespace KJS {

static const char SVGPathSegLinetoVerticalRelImpl__s_hashTableStrings[] = {
    "\0"
    "y\0"
};


static const struct HashEntry SVGPathSegLinetoVerticalRelImpl__s_hashTableEntries[] = {
   { 0, 0, 0, 0, -1 },
   { 1, SVGPathSegLinetoVerticalRelImpl::Y, DontDelete, 0, -1 }
};

const struct HashTable SVGPathSegLinetoVerticalRelImpl::s_hashTable = { 2, 2, SVGPathSegLinetoVerticalRelImpl__s_hashTableEntries, 2, SVGPathSegLinetoVerticalRelImpl__s_hashTableStrings};

} // namespace
