/* Automatically generated from impl/SVGPatternElementImpl.cc using ../../kdelibs/kjs/create_hash_table. DO NOT EDIT ! */

using namespace KJS;

namespace KJS {

static const char SVGPatternElementImpl__s_hashTableStrings[] = {
    "\0"
    "patternContentUnits\0"
    "patternTransform\0"
    "patternUnits\0"
    "height\0"
    "width\0"
    "x\0"
    "y\0"
};


static const struct HashEntry SVGPatternElementImpl__s_hashTableEntries[] = {
   { 66, SVGPatternElementImpl::Y, DontDelete|ReadOnly, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 1, SVGPatternElementImpl::PatternContentUnits, DontDelete|ReadOnly, 0, -1 },
   { 58, SVGPatternElementImpl::Width, DontDelete|ReadOnly, 0, -1 },
   { 51, SVGPatternElementImpl::Height, DontDelete|ReadOnly, 0, 12 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 0, 0, 0, 0, -1 },
   { 64, SVGPatternElementImpl::X, DontDelete|ReadOnly, 0, 11 },
   { 38, SVGPatternElementImpl::PatternUnits, DontDelete|ReadOnly, 0, -1 },
   { 21, SVGPatternElementImpl::PatternTransform, DontDelete|ReadOnly, 0, -1 }
};

const struct HashTable SVGPatternElementImpl::s_hashTable = { 2, 13, SVGPatternElementImpl__s_hashTableEntries, 11, SVGPatternElementImpl__s_hashTableStrings};

} // namespace
